@@ -17,11 +17,11 @@ FCGI::Buffer - Verify, Cache and Optimise FCGI Output
 
 =head1 VERSION
 
-Version 0.05
+Version 0.06
 
 =cut
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 =head1 SYNOPSIS
 
@@ -298,33 +298,22 @@ sub DESTROY {
 	# Generate the eTag before compressing, since the compressed data
 	# includes the mtime field which changes thus causing a different
 	# Etag to be generated
-	my $encode_loaded;
 	if($ENV{'SERVER_PROTOCOL'} &&
 	  ($ENV{'SERVER_PROTOCOL'} eq 'HTTP/1.1') &&
 	  $self->{generate_etag} && defined($self->{body})) {
 		# encode to avoid "Wide character in subroutine entry"
 		require Encode;
-		$encode_loaded = 1;
+		$self->{_encode_loaded} = 1;
 		$self->{etag} = '"' . Digest::MD5->new->add(Encode::encode_utf8($self->{body}))->hexdigest() . '"';
 		if($ENV{'HTTP_IF_NONE_MATCH'} && $self->{generate_304} && ($self->{status} == 200)) {
-			if($self->{logger}) {
-				$self->{logger}->debug("Compare $ENV{HTTP_IF_NONE_MATCH} with $self->{etag}");
-			}
-			if($ENV{'HTTP_IF_NONE_MATCH'} eq $self->{etag}) {
-				push @{$self->{o}}, "Status: 304 Not Modified";
-				$self->{send_body} = 0;
-				$self->{status} = 304;
-				if($self->{logger}) {
-					$self->{logger}->debug('Set status to 304');
-				}
-			}
+			$self->_check_if_none_match();
 		}
 	}
 
 	my $encoding = $self->_should_gzip();
 	my $unzipped_body = $self->{body};
 
-	if(defined($self->{body})) {
+	if(defined($unzipped_body)) {
 		my $range = $ENV{'Range'} ? $ENV{'Range'} : $ENV{'HTTP_RANGE'};
 
 		if($range && !$self->{cache}) {
@@ -340,22 +329,7 @@ sub DESTROY {
 				$unzipped_body = $self->{body};
 			}
 		}
-		if((length($encoding) > 0) && (length($self->{body}) >= MIN_GZIP_LEN)) {
-			require Compress::Zlib;
-			Compress::Zlib->import;
-
-			# Avoid 'Wide character in memGzip'
-			unless($encode_loaded) {
-				require Encode;
-				$encode_loaded = 1;
-			}
-			my $nbody = Compress::Zlib::memGzip(\Encode::encode_utf8($self->{body}));
-			if(length($nbody) < length($self->{body})) {
-				$self->{body} = $nbody;
-				push @{$self->{o}}, "Content-Encoding: $encoding";
-				push @{$self->{o}}, "Vary: Accept-Encoding";
-			}
-		}
+		$self->_compress({ encoding => $encoding });
 	}
 
 	if($self->{cache}) {
@@ -424,47 +398,22 @@ sub DESTROY {
 			  ($self->{status} == 200)) {
 				if($ENV{'HTTP_IF_NONE_MATCH'} && $self->{generate_etag}) {
 					if(!defined($self->{etag})) {
-						unless($encode_loaded) {
+						unless($self->{_encode_loaded}) {
 							require Encode;
-							$encode_loaded = 1;
+							$self->{_encode_loaded} = 1;
 						}
 						$self->{etag} = '"' . Digest::MD5->new->add(Encode::encode_utf8($self->{body}))->hexdigest() . '"';
 					}
-					if($self->{'logger'} && $self->{generate_304}) {
-						$self->{logger}->debug("Compare etags $ENV{HTTP_IF_NONE_MATCH} and $self->{etag}");
-					}
-					if(($ENV{'HTTP_IF_NONE_MATCH'} eq $self->{etag}) && $self->{generate_304}) {
-						push @{$self->{o}}, "Status: 304 Not Modified";
-						$self->{status} = 304;
-						$self->{send_body} = 0;
-						if($self->{logger}) {
-							$self->{logger}->debug('Set status to 304');
-						}
-					}
+					$self->_check_if_none_match();
 				}
 			}
 			if($self->{status} == 200) {
 				$encoding = $self->_should_gzip();
-				if($self->{send_body} && (length($encoding) > 0)) {
-					if(length($self->{body}) >= MIN_GZIP_LEN) {
-						require Compress::Zlib;
-						Compress::Zlib->import;
-
-						# Avoid 'Wide character in memGzip'
-						unless($encode_loaded) {
-							require Encode;
-							$encode_loaded = 1;
-						}
-						if($self->{generate_etag} && !defined($self->{etag}) && ((!defined($headers)) || ($headers !~ /^ETag: /m))) {
-							$self->{etag} = '"' . Digest::MD5->new->add(Encode::encode_utf8($self->{body}))->hexdigest() . '"';
-						}
-						my $nbody = Compress::Zlib::memGzip(\Encode::encode_utf8($self->{body}));
-						if(length($nbody) < length($self->{body})) {
-							$self->{body} = $nbody;
-							push @{$self->{o}}, "Content-Encoding: $encoding";
-							push @{$self->{o}}, "Vary: Accept-Encoding";
-						}
+				if($self->{send_body}) {
+					if($self->{generate_etag} && !defined($self->{etag}) && ((!defined($headers)) || ($headers !~ /^ETag: /m))) {
+						$self->{etag} = '"' . Digest::MD5->new->add(Encode::encode_utf8($self->{body}))->hexdigest() . '"';
 					}
+					$self->_compress({ encoding => $encoding });
 				}
 			}
 			my $cannot_304 = !$self->{generate_304};
@@ -476,17 +425,7 @@ sub DESTROY {
 				}
 			}
 			if($ENV{'HTTP_IF_NONE_MATCH'} && $self->{send_body} && ($self->{status} != 304) && $self->{generate_304}) {
-				if($self->{logger}) {
-					$self->{logger}->debug("Compare $ENV{HTTP_IF_NONE_MATCH} with $self->{etag}");
-				}
-				if(defined($self->{etag}) && ($self->{etag} eq $ENV{'HTTP_IF_NONE_MATCH'}) && ($self->{status} == 200)) {
-					push @{$self->{o}}, "Status: 304 Not Modified";
-					$self->{send_body} = 0;
-					$self->{status} = 304;
-					if($self->{logger}) {
-						$self->{logger}->debug('Set status to 304');
-					}
-				} else {
+				if(!$self->_check_if_none_match()) {
 					$cannot_304 = 1;
 				}
 			}
@@ -499,12 +438,13 @@ sub DESTROY {
 				}
 				if(($self->{status} == 200) && $self->{generate_last_modified}) {
 					if($self->{logger}) {
-						$self->{logger}->debug('Set Last-Modified to ' . HTTP::Date::time2str($self->{cobject}->created_at()));
+						$self->{logger}->debug('Set Last-Modified to ', HTTP::Date::time2str($self->{cobject}->created_at()));
 					}
 					push @{$self->{o}}, "Last-Modified: " . HTTP::Date::time2str($self->{cobject}->created_at());
 				}
 			}
 		} else {
+			# Not in the server side cache
 			if($self->{status} == 200) {
 				unless($self->{cache_age}) {
 					# It would be great if CHI::set()
@@ -542,10 +482,10 @@ sub DESTROY {
 				# TODO: Support the Expires header
 				# if($headers !~ /^Expires: /m))) {
 				# }
-				$self->{cache}->set($key, Storable::freeze($cache_hash), $self->{cache_age});
 				if($self->{logger}) {
-					$self->{logger}->debug("store $key in the cache");
+					$self->{logger}->debug("store $key in the cache, age = ", $self->{cache_age});
 				}
+				$self->{cache}->set($key, Storable::freeze($cache_hash), $self->{cache_age});
 				if($self->{generate_last_modified}) {
 					$self->{cobject} = $self->{cache}->get_object($key);
 					if(defined($self->{cobject})) {
@@ -725,7 +665,7 @@ sub _optimise_content {
 	my $self = shift;
 
 	# Regexp::List - wow!
-	$self->{body} =~ s/(?^:(?:(?:\s+|\r)\n|\n(?:\s+|\+)))/\n/g;
+	$self->{body} =~ s/(((\s+|\r)\n|\n(\s+|\+)))/\n/g;
 	# $self->{body} =~ s/\r\n/\n/gs;
 	# $self->{body} =~ s/\s+\n/\n/gs;
 	# $self->{body} =~ s/\n+/\n/gs;
@@ -766,10 +706,11 @@ sub _optimise_content {
 	$self->{body} =~ s/(\s?<hr>\s+|\s+<hr>\s?)/<hr>/gis;
 	# $self->{body} =~ s/\s<hr>/<hr>/gis;
 	# $self->{body} =~ s/<hr>\s/<hr>/gis;
-	$self->{body} =~ s/<\/li>\+s<li>/<\/li><li>/gis;
-	$self->{body} =~ s/<\/li>\+s<\/ul>/<\/li><\/ul>/gis;
-	$self->{body} =~ s/<ul>\+s<li>/<ul><li>/gis;
+	$self->{body} =~ s/<\/li>\s+<li>/<\/li><li>/gis;
+	$self->{body} =~ s/<\/li>\s+<\/ul>/<\/li><\/ul>/gis;
+	$self->{body} =~ s/<ul>\s+<li>/<ul><li>/gis;
 	$self->{body} =~ s/\<\/option\>\s+\<option/\<\/option\>\<option/gis;
+	$self->{body} =~ s/<title>\s*(.+?)\s*<\/title>/<title>$1<\/title>/is;
 }
 
 # Create a key for the cache
@@ -787,7 +728,6 @@ sub _generate_key {
 	my $key = $self->{info}->browser_type() . '::' . $self->{info}->domain_name() . '::' . $self->{info}->script_name() . '::' . $self->{info}->as_string();
 	if($ENV{'HTTP_COOKIE'}) {
 		# Different states of the client are stored in different caches
-		my $cookies = $ENV{'HTTP_COOKIE'};
 		# Don't put different Google Analytics in different caches, and anyway they
 		# would be wrong
 		foreach my $cookie(split(/;/, $ENV{'HTTP_COOKIE'})) {
@@ -1164,6 +1104,64 @@ sub _set_content_type {
 	}
 }
 
+sub _compress()
+{
+	my $self = shift;
+	my %params = (ref($_[0]) eq 'HASH') ? %{$_[0]} : @_;
+
+	my $encoding = $params{encoding};
+
+	if((length($encoding) == 0) || (length($self->{body}) < MIN_GZIP_LEN)) {
+		return;
+	}
+	require Compress::Zlib;
+	Compress::Zlib->import;
+
+	# Avoid 'Wide character in memGzip'
+	unless($self->{_encode_loaded}) {
+		require Encode;
+		$self->{_encode_loaded} = 1;
+	}
+	my $nbody = Compress::Zlib::memGzip(\Encode::encode_utf8($self->{body}));
+	if(length($nbody) < length($self->{body})) {
+		$self->{body} = $nbody;
+		push @{$self->{o}}, "Content-Encoding: $encoding";
+		push @{$self->{o}}, "Vary: Accept-Encoding";
+	}
+}
+
+sub _check_if_none_match {
+	my $self = shift;
+
+	if($self->{logger}) {
+		$self->{logger}->debug("Compare $ENV{HTTP_IF_NONE_MATCH} with $self->{etag}");
+	}
+	if($ENV{'HTTP_IF_NONE_MATCH'} eq $self->{etag}) {
+		push @{$self->{o}}, "Status: 304 Not Modified";
+		$self->{send_body} = 0;
+		$self->{status} = 304;
+		if($self->{logger}) {
+			$self->{logger}->debug('Set status to 304');
+		}
+		return 1;
+	}
+	if($self->{cache} && $self->{logger} && $self->{logger}->is_debug()) {
+		my $cached_copy = $self->{cache}->get($self->_generate_key());
+
+		if($cached_copy && $self->{body}) {
+			require Text::Diff;
+			Text::Diff->import();
+
+			$cached_copy = Storable::thaw($cached_copy)->{body};
+			my $diffs = diff(\$self->{body}, \$cached_copy);
+			$self->{logger}->debug('diffs: ', $diffs);
+		} else {
+			$self->{logger}->debug('Nothing to compare');
+		}
+	}
+	return 0;
+}
+
 =head1 AUTHOR
 
 Nigel Horne, C<< <njh at bandsman.co.uk> >>
